@@ -18,6 +18,8 @@ package org.apache.catalina.ssi;
 
 
 import java.io.PrintWriter;
+
+import org.apache.tomcat.util.res.StringManager;
 /**
  * Implements the Server-side #exec command
  *
@@ -27,6 +29,7 @@ import java.io.PrintWriter;
  * @author David Becker
  */
 public final class SSIConfig implements SSICommand {
+    private static final StringManager sm = StringManager.getManager(SSIConfig.class);
     /**
      * @see SSICommand
      */
@@ -45,7 +48,7 @@ public final class SSIConfig implements SSICommand {
             } else if (paramName.equalsIgnoreCase("timefmt")) {
                 ssiMediator.setConfigTimeFmt(substitutedValue);
             } else {
-                ssiMediator.log("#config--Invalid attribute: " + paramName);
+                ssiMediator.log(sm.getString("ssiCommand.invalidAttribute", paramName));
                 //We need to fetch this value each time, since it may change
                 // during the
                 // loop
