@@ -39,6 +39,7 @@ import javax.management.ObjectName;
 import javax.servlet.http.HttpUpgradeHandler;
 import javax.servlet.http.WebConnection;
 
+import org.apache.coyote.http11.upgrade.InternalHttpUpgradeHandler;
 import org.apache.juli.logging.Log;
 import org.apache.tomcat.InstanceManager;
 import org.apache.tomcat.util.ExceptionUtils;
@@ -95,9 +96,9 @@ public abstract class AbstractProtocol<S> implements ProtocolHandler,
             Collections.newSetFromMap(new ConcurrentHashMap<Processor, Boolean>());
 
     /**
-     * Controller for the async timeout scheduling.
+     * Controller for the timeout scheduling.
      */
-    private ScheduledFuture<?> asyncTimeoutFuture = null;
+    private ScheduledFuture<?> timeoutFuture = null;
     private ScheduledFuture<?> monitorFuture;
 
     public AbstractProtocol(AbstractEndpoint<S,?> endpoint) {
@@ -310,11 +311,27 @@ public abstract class AbstractProtocol<S> implements ProtocolHandler,
         return endpoint.getConnectionCount();
     }
 
+    /**
+     * NO-OP.
+     *
+     * @param threadCount Unused
+     *
+     * @deprecated Will be removed in Tomcat 10.
+     */
+    @Deprecated
     public void setAcceptorThreadCount(int threadCount) {
-        endpoint.setAcceptorThreadCount(threadCount);
     }
+
+    /**
+     * Always returns 1.
+     *
+     * @return Always 1.
+     *
+     * @deprecated Will be removed in Tomcat 10.
+     */
+    @Deprecated
     public int getAcceptorThreadCount() {
-      return endpoint.getAcceptorThreadCount();
+      return 1;
     }
 
     public void setAcceptorThreadPriority(int threadPriority) {
@@ -578,17 +595,22 @@ public abstract class AbstractProtocol<S> implements ProtocolHandler,
     }
 
 
+    /**
+     * Note: The name of this method originated with the Servlet 3.0
+     * asynchronous processing but evolved over time to represent a timeout that
+     * is triggered independently of the socket read/write timeouts.
+     */
     protected void startAsyncTimeout() {
-        if (asyncTimeoutFuture == null || (asyncTimeoutFuture != null && asyncTimeoutFuture.isDone())) {
-            if (asyncTimeoutFuture != null && asyncTimeoutFuture.isDone()) {
+        if (timeoutFuture == null || (timeoutFuture != null && timeoutFuture.isDone())) {
+            if (timeoutFuture != null && timeoutFuture.isDone()) {
                 // There was an error executing the scheduled task, get it and log it
                 try {
-                    asyncTimeoutFuture.get();
+                    timeoutFuture.get();
                 } catch (InterruptedException | ExecutionException e) {
                     getLog().error(sm.getString("abstractProtocolHandler.asyncTimeoutError"), e);
                 }
             }
-            asyncTimeoutFuture = getUtilityExecutor().scheduleAtFixedRate(
+            timeoutFuture = getUtilityExecutor().scheduleAtFixedRate(
                     new Runnable() {
                         @Override
                         public void run() {
@@ -602,9 +624,9 @@ public abstract class AbstractProtocol<S> implements ProtocolHandler,
     }
 
     protected void stopAsyncTimeout() {
-        if (asyncTimeoutFuture != null) {
-            asyncTimeoutFuture.cancel(false);
-            asyncTimeoutFuture = null;
+        if (timeoutFuture != null) {
+            timeoutFuture.cancel(false);
+            timeoutFuture = null;
         }
     }
 
@@ -647,7 +669,7 @@ public abstract class AbstractProtocol<S> implements ProtocolHandler,
             monitorFuture = null;
         }
         stopAsyncTimeout();
-        // Timeout any pending async request
+        // Timeout any waiting processor
         for (Processor processor : waitingProcessors) {
             processor.timeoutAsync(-1);
         }
@@ -753,12 +775,14 @@ public abstract class AbstractProtocol<S> implements ProtocolHandler,
                         processor, socket));
             }
 
-            // Async timeouts are calculated on a dedicated thread and then
+            // Timeouts are calculated on a dedicated thread and then
             // dispatched. Because of delays in the dispatch process, the
             // timeout may no longer be required. Check here and avoid
             // unnecessary processing.
-            if (SocketEvent.TIMEOUT == status && (processor == null ||
-                    !processor.isAsync() || !processor.checkAsyncTimeoutGeneration())) {
+            if (SocketEvent.TIMEOUT == status &&
+                    (processor == null ||
+                    !processor.isAsync() && !processor.isUpgrade() ||
+                    processor.isAsync() && !processor.checkAsyncTimeoutGeneration())) {
                 // This is effectively a NO-OP
                 return SocketState.OPEN;
             }
@@ -777,7 +801,9 @@ public abstract class AbstractProtocol<S> implements ProtocolHandler,
             try {
                 if (processor == null) {
                     String negotiatedProtocol = wrapper.getNegotiatedProtocol();
-                    if (negotiatedProtocol != null) {
+                    // OpenSSL typically returns null whereas JSSE typically
+                    // returns "" when no protocol is negotiated
+                    if (negotiatedProtocol != null && negotiatedProtocol.length() > 0) {
                         UpgradeProtocol upgradeProtocol =
                                 getProtocol().getNegotiatedProtocol(negotiatedProtocol);
                         if (upgradeProtocol != null) {
@@ -886,6 +912,12 @@ public abstract class AbstractProtocol<S> implements ProtocolHandler,
                                     upgradeToken.getContextBind().unbind(false, oldCL);
                                 }
                             }
+                            if (httpUpgradeHandler instanceof InternalHttpUpgradeHandler) {
+                                if (((InternalHttpUpgradeHandler) httpUpgradeHandler).hasAsyncIO()) {
+                                    // The handler will initiate all further I/O
+                                    state = SocketState.LONG;
+                                }
+                            }
                         }
                     }
                 } while ( state == SocketState.UPGRADING);
@@ -917,6 +949,7 @@ public abstract class AbstractProtocol<S> implements ProtocolHandler,
                     // to the poller if necessary.
                     if (status != SocketEvent.OPEN_WRITE) {
                         longPoll(wrapper, processor);
+                        getProtocol().addWaitingProcessor(processor);
                     }
                 } else if (state == SocketState.SUSPENDED) {
                     // Don't add sockets back to the poller.
@@ -968,7 +1001,13 @@ public abstract class AbstractProtocol<S> implements ProtocolHandler,
             // Future developers: if you discover any other
             // rare-but-nonfatal exceptions, catch them here, and log as
             // above.
-            catch (Throwable e) {
+            catch (OutOfMemoryError oome) {
+                // Try and handle this here to give Tomcat a chance to close the
+                // connection and prevent clients waiting until they time out.
+                // Worst case, it isn't recoverable and the attempt at logging
+                // will trigger another OOME.
+                getLog().error(sm.getString("abstractConnectionHandler.oome"), oome);
+            } catch (Throwable e) {
                 ExceptionUtils.handleThrowable(e);
                 // any other exception or error is odd. Here we log it
                 // with "ERROR" level, so it will show up even on
