@@ -24,18 +24,20 @@ import java.util.concurrent.ThreadFactory;
 import java.util.concurrent.TimeUnit;
 
 /**
- * Provides a shared idle object eviction timer for all pools. This class is
- * currently implemented using {@link ScheduledThreadPoolExecutor}. This
- * implementation may change in any future release. This class keeps track of
- * how many pools are using it. If no pools are using the timer, it is cancelled.
- * This prevents a thread being left running which, in application server
- * environments, can lead to memory leads and/or prevent applications from
- * shutting down or reloading cleanly.
+ * Provides a shared idle object eviction timer for all pools.
  * <p>
- * This class has package scope to prevent its inclusion in the pool public API.
- * The class declaration below should *not* be changed to public.
+ * This class is currently implemented using {@link ScheduledThreadPoolExecutor}. This implementation may change in any
+ * future release. This class keeps track of how many pools are using it. If no pools are using the timer, it is
+ * cancelled. This prevents a thread being left running which, in application server environments, can lead to memory
+ * leads and/or prevent applications from shutting down or reloading cleanly.
+ * </p>
+ * <p>
+ * This class has package scope to prevent its inclusion in the pool public API. The class declaration below should
+ * *not* be changed to public.
+ * </p>
  * <p>
  * This class is intended to be thread-safe.
+ * </p>
  *
  * @since 2.0
  */
@@ -44,7 +46,7 @@ class EvictionTimer {
     /** Executor instance */
     private static ScheduledThreadPoolExecutor executor; //@GuardedBy("EvictionTimer.class")
 
-    /** Prevent instantiation */
+    /** Prevents instantiation */
     private EvictionTimer() {
         // Hide the default constructor
     }
@@ -62,13 +64,14 @@ class EvictionTimer {
 
 
     /**
-     * Add the specified eviction task to the timer. Tasks that are added with a
+     * Adds the specified eviction task to the timer. Tasks that are added with a
      * call to this method *must* call {@link #cancel(BaseGenericObjectPool.Evictor,long,TimeUnit)}
      * to cancel the task to prevent memory and/or thread leaks in application
      * server environments.
-     * @param task      Task to be scheduled
-     * @param delay     Delay in milliseconds before task is executed
-     * @param period    Time in milliseconds between executions
+     *
+     * @param task      Task to be scheduled.
+     * @param delay     Delay in milliseconds before task is executed.
+     * @param period    Time in milliseconds between executions.
      */
     static synchronized void schedule(
             final BaseGenericObjectPool<?>.Evictor task, final long delay, final long period) {
@@ -82,18 +85,20 @@ class EvictionTimer {
     }
 
     /**
-     * Remove the specified eviction task from the timer.
+     * Removes the specified eviction task from the timer.
      *
-     * @param task      Task to be cancelled
+     * @param evictor   Task to be cancelled.
      * @param timeout   If the associated executor is no longer required, how
      *                  long should this thread wait for the executor to
      *                  terminate?
-     * @param unit      The units for the specified timeout
+     * @param unit      The units for the specified timeout.
      */
     static synchronized void cancel(
-            final BaseGenericObjectPool<?>.Evictor task, final long timeout, final TimeUnit unit) {
-        task.cancel();
-        if (executor.getQueue().size() == 0) {
+            final BaseGenericObjectPool<?>.Evictor evictor, final long timeout, final TimeUnit unit) {
+        if (evictor != null) {
+            evictor.cancel();
+        }
+        if (executor != null && executor.getQueue().isEmpty()) {
             executor.shutdown();
             try {
                 executor.awaitTermination(timeout, unit);
@@ -107,23 +112,23 @@ class EvictionTimer {
     }
 
     /**
-     * Thread factory that creates a thread, with the context classloader from this class.
+     * Thread factory that creates a daemon thread, with the context class loader from this class.
      */
     private static class EvictorThreadFactory implements ThreadFactory {
 
         @Override
-        public Thread newThread(final Runnable r) {
-            final Thread t = new Thread(null, r, "commons-pool-evictor-thread");
-
+        public Thread newThread(final Runnable runnable) {
+            final Thread thread = new Thread(null, runnable, "commons-pool-evictor-thread");
+            thread.setDaemon(true); // POOL-363 - Required for applications using Runtime.addShutdownHook().
             AccessController.doPrivileged(new PrivilegedAction<Void>() {
                 @Override
                 public Void run() {
-                    t.setContextClassLoader(EvictorThreadFactory.class.getClassLoader());
+                    thread.setContextClassLoader(EvictorThreadFactory.class.getClassLoader());
                     return null;
                 }
             });
 
-            return t;
+            return thread;
         }
     }
 }
