@@ -17,7 +17,6 @@
 package org.apache.tomcat.dbcp.pool2.impl;
 
 import java.util.ArrayList;
-import java.util.Collection;
 import java.util.Deque;
 import java.util.HashMap;
 import java.util.Iterator;
@@ -51,12 +50,10 @@ import org.apache.tomcat.dbcp.pool2.PooledObjectState;
  * {@link #borrowObject borrowObject} methods. Each time a new key value is
  * provided to one of these methods, a sub-new pool is created under the given
  * key to be managed by the containing <code>GenericKeyedObjectPool.</code>
- * </p>
  * <p>
  * Note that the current implementation uses a ConcurrentHashMap which uses
  * equals() to compare keys.
  * This means that distinct instance keys must be distinguishable using equals.
- * </p>
  * <p>
  * Optionally, one may configure the pool to examine and possibly evict objects
  * as they sit idle in the pool and to ensure that a minimum number of idle
@@ -65,15 +62,12 @@ import org.apache.tomcat.dbcp.pool2.PooledObjectState;
  * configuring this optional feature. Eviction runs contend with client threads
  * for access to objects in the pool, so if they run too frequently performance
  * issues may result.
- * </p>
  * <p>
  * Implementation note: To prevent possible deadlocks, care has been taken to
  * ensure that no call to a factory method will occur within a synchronization
  * block. See POOL-125 and DBCP-44 for more information.
- * </p>
  * <p>
  * This class is intended to be thread-safe.
- * </p>
  *
  * @see GenericObjectPool
  *
@@ -453,11 +447,6 @@ public class GenericKeyedObjectPool<K, T> extends BaseGenericObjectPool<T>
 
         final ObjectDeque<T> objectDeque = poolMap.get(key);
 
-        if (objectDeque == null) {
-            throw new IllegalStateException(
-                    "No keyed pool found under the given key.");
-        }
-
         final PooledObject<T> p = objectDeque.getAllObjects().get(new IdentityWrapper<>(obj));
 
         if (p == null) {
@@ -1088,16 +1077,8 @@ public class GenericKeyedObjectPool<K, T> extends BaseGenericObjectPool<T>
         final ObjectDeque<T> objectDeque = register(key);
 
         try {
-            boolean isIdle;
-            synchronized(toDestroy) {
-                // Check idle state directly
-                isIdle = toDestroy.getState().equals(PooledObjectState.IDLE);
-                // If idle, not under eviction test, or always is true, remove instance,
-                // updating isIdle if instance is found in idle objects
-                if (isIdle || always) {
-                    isIdle = objectDeque.getIdleObjects().remove(toDestroy);
-                }
-            }
+            final boolean isIdle = objectDeque.getIdleObjects().remove(toDestroy);
+
             if (isIdle || always) {
                 objectDeque.getAllObjects().remove(new IdentityWrapper<>(toDestroy.getObject()));
                 toDestroy.invalidate();
@@ -1170,9 +1151,10 @@ public class GenericKeyedObjectPool<K, T> extends BaseGenericObjectPool<T>
      */
     private void deregister(final K k) {
         Lock lock = keyLock.readLock();
+        ObjectDeque<T> objectDeque;
         try {
             lock.lock();
-            final ObjectDeque<T> objectDeque = poolMap.get(k);
+            objectDeque = poolMap.get(k);
             final long numInterested = objectDeque.getNumInterested().decrementAndGet();
             if (numInterested == 0 && objectDeque.getCreateCount().get() == 0) {
                 // Potential to remove key
@@ -1261,58 +1243,6 @@ public class GenericKeyedObjectPool<K, T> extends BaseGenericObjectPool<T>
         }
     }
 
-    /**
-     * Calls {@link KeyedObjectPool#addObject(Object)} with each
-     * key in <code>keys</code> for <code>count</code> number of times. This has
-     * the same effect as calling {@link #addObjects(Object, int)}
-     * for each key in the <code>keys</code> collection.
-     *
-     * @param keys
-     *            {@link Collection} of keys to add objects for.
-     * @param count
-     *            the number of idle objects to add for each <code>key</code>.
-     * @throws Exception
-     *             when {@link KeyedObjectPool#addObject(Object)} fails.
-     * @throws IllegalArgumentException
-     *             when <code>keyedPool</code>, <code>keys</code>, or any value
-     *             in <code>keys</code> is <code>null</code>.
-     * @see #addObjects(Object, int)
-     */
-    @Override
-    public void addObjects(final Collection<K> keys, final int count) throws Exception, IllegalArgumentException {
-        if (keys == null) {
-            throw new IllegalArgumentException(PoolUtils.MSG_NULL_KEYS);
-        }
-        final Iterator<K> iter = keys.iterator();
-        while (iter.hasNext()) {
-            addObjects(iter.next(), count);
-        }
-    }
-
-    /**
-     * Calls {@link KeyedObjectPool#addObject(Object)}
-     * <code>key</code> <code>count</code> number of times.
-     *
-     * @param key
-     *            the key to add objects for.
-     * @param count
-     *            the number of idle objects to add for <code>key</code>.
-     * @throws Exception
-     *             when {@link KeyedObjectPool#addObject(Object)} fails.
-     * @throws IllegalArgumentException
-     *             when <code>key</code> is <code>null</code>.
-     * @since 2.8.0
-     */
-    @Override
-    public void addObjects(final K key, final int count) throws Exception, IllegalArgumentException {
-        if (key == null) {
-            throw new IllegalArgumentException(PoolUtils.MSG_NULL_KEY);
-        }
-        for (int i = 0; i < count; i++) {
-            addObject(key);
-        }
-    }
-
     /**
      * Add an object to the set of idle objects for a given key.
      *
