@@ -101,7 +101,8 @@ public class Host {
         @Override
         public int read(char[] cbuf, int off, int len) throws IOException {
             for (int i = off; i < off + len; i++) {
-                cbuf[i] = (char) bytes[pos++];
+                // Want output in range 0 to 255, not -128 to 127
+                cbuf[i] = (char) (bytes[pos++] & 0xFF);
             }
             return len;
         }
@@ -116,7 +117,8 @@ public class Host {
         @Override
         public int read() throws IOException {
             if (pos < end) {
-                return bytes[pos++];
+                // Want output in range 0 to 255, not -128 to 127
+                return bytes[pos++] & 0xFF;
             } else {
                 return -1;
             }
