@@ -22,11 +22,9 @@ import java.io.IOException;
 import java.io.Serializable;
 import java.security.KeyStore;
 import java.security.UnrecoverableKeyException;
-import java.util.EnumMap;
 import java.util.HashSet;
 import java.util.LinkedHashSet;
 import java.util.List;
-import java.util.Map;
 import java.util.Set;
 
 import javax.management.ObjectName;
@@ -65,8 +63,6 @@ public class SSLHostConfig implements Serializable {
     }
 
     private Type configType = null;
-    private Type currentConfigType = null;
-    private Map<Type, Set<String>> configuredProperties = new EnumMap<>(Type.class);
 
     private String hostName = DEFAULT_SSL_HOST_NAME;
 
@@ -88,7 +84,7 @@ public class SSLHostConfig implements Serializable {
     private Set<String> explicitlyRequestedProtocols = new HashSet<>();
     // Nested
     private SSLHostConfigCertificate defaultCertificate = null;
-    private Set<SSLHostConfigCertificate> certificates = new HashSet<>(4);
+    private Set<SSLHostConfigCertificate> certificates = new LinkedHashSet<>(4);
     // Common
     private String certificateRevocationListFile;
     private CertificateVerification certificateVerification = CertificateVerification.NONE;
@@ -100,11 +96,12 @@ public class SSLHostConfig implements Serializable {
     private List<String> jsseCipherNames = null;
     private boolean honorCipherOrder = false;
     private Set<String> protocols = new HashSet<>();
+    // Values <0 mean use the implementation default
+    private int sessionCacheSize = -1;
+    private int sessionTimeout = 86400;
     // JSSE
     private String keyManagerAlgorithm = KeyManagerFactory.getDefaultAlgorithm();
     private boolean revocationEnabled = false;
-    private int sessionCacheSize = 0;
-    private int sessionTimeout = 86400;
     private String sslProtocol = Constants.SSL_PROTO_TLS;
     private String trustManagerClassName;
     private String truststoreAlgorithm = TrustManagerFactory.getDefaultAlgorithm();
@@ -152,39 +149,11 @@ public class SSLHostConfig implements Serializable {
     public String getConfigType() {
         return configType.name();
     }
-    public void setConfigType(Type configType) {
-        this.configType = configType;
-        if (configType == Type.EITHER) {
-            if (configuredProperties.remove(Type.JSSE) == null) {
-                configuredProperties.remove(Type.OPENSSL);
-            }
-        } else {
-            configuredProperties.remove(configType);
-        }
-        for (Map.Entry<Type,Set<String>> entry : configuredProperties.entrySet()) {
-            for (String property : entry.getValue()) {
-                log.warn(sm.getString("sslHostConfig.mismatch",
-                        property, getHostName(), entry.getKey(), configType));
-            }
-        }
-    }
 
 
     void setProperty(String name, Type configType) {
         if (this.configType == null) {
-            Set<String> properties = configuredProperties.get(configType);
-            if (properties == null) {
-                properties = new HashSet<>();
-                configuredProperties.put(configType, properties);
-            }
-            properties.add(name);
-        } else if (this.configType == Type.EITHER) {
-            if (currentConfigType == null) {
-                currentConfigType = configType;
-            } else if (currentConfigType != configType) {
-                log.warn(sm.getString("sslHostConfig.mismatch",
-                        name, getHostName(), configType, currentConfigType));
-            }
+            this.configType = configType;
         } else {
             if (configType != this.configType) {
                 log.warn(sm.getString("sslHostConfig.mismatch",
@@ -411,7 +380,7 @@ public class SSLHostConfig implements Serializable {
 
     public LinkedHashSet<Cipher> getCipherList() {
         if (cipherList == null) {
-            cipherList = OpenSSLCipherConfigurationParser.parse(ciphers);
+            cipherList = OpenSSLCipherConfigurationParser.parse(getCiphers());
         }
         return cipherList;
     }
@@ -517,6 +486,26 @@ public class SSLHostConfig implements Serializable {
     }
 
 
+    public void setSessionCacheSize(int sessionCacheSize) {
+        this.sessionCacheSize = sessionCacheSize;
+    }
+
+
+    public int getSessionCacheSize() {
+        return sessionCacheSize;
+    }
+
+
+    public void setSessionTimeout(int sessionTimeout) {
+        this.sessionTimeout = sessionTimeout;
+    }
+
+
+    public int getSessionTimeout() {
+        return sessionTimeout;
+    }
+
+
     // ---------------------------------- JSSE specific configuration properties
 
     // TODO: These certificate setters can be removed once it is no longer
@@ -594,28 +583,6 @@ public class SSLHostConfig implements Serializable {
     }
 
 
-    public void setSessionCacheSize(int sessionCacheSize) {
-        setProperty("sessionCacheSize", Type.JSSE);
-        this.sessionCacheSize = sessionCacheSize;
-    }
-
-
-    public int getSessionCacheSize() {
-        return sessionCacheSize;
-    }
-
-
-    public void setSessionTimeout(int sessionTimeout) {
-        setProperty("sessionTimeout", Type.JSSE);
-        this.sessionTimeout = sessionTimeout;
-    }
-
-
-    public int getSessionTimeout() {
-        return sessionTimeout;
-    }
-
-
     public void setSslProtocol(String sslProtocol) {
         setProperty("sslProtocol", Type.JSSE);
         this.sslProtocol = sslProtocol;
@@ -872,8 +839,7 @@ public class SSLHostConfig implements Serializable {
 
     public enum Type {
         JSSE,
-        OPENSSL,
-        EITHER
+        OPENSSL
     }
 
 
