@@ -29,6 +29,9 @@ import java.util.concurrent.RejectedExecutionException;
 import java.util.concurrent.Semaphore;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicBoolean;
+import java.util.concurrent.locks.Lock;
+import java.util.concurrent.locks.ReentrantReadWriteLock;
+import java.util.concurrent.locks.ReentrantReadWriteLock.WriteLock;
 
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
@@ -41,8 +44,10 @@ public abstract class SocketWrapperBase<E> {
 
     protected static final StringManager sm = StringManager.getManager(SocketWrapperBase.class);
 
-    private final E socket;
-    private final AbstractEndpoint<E> endpoint;
+    private E socket;
+    private final AbstractEndpoint<E,?> endpoint;
+
+    protected final AtomicBoolean closed = new AtomicBoolean(false);
 
     // Volatile because I/O and setting the timeout values occurs on a different
     // thread to the thread checking the timeout.
@@ -53,7 +58,6 @@ public abstract class SocketWrapperBase<E> {
     private volatile boolean upgraded = false;
     private boolean secure = false;
     private String negotiatedProtocol = null;
-
     /*
      * Following cached for speed / reduced GC
      */
@@ -63,8 +67,14 @@ public abstract class SocketWrapperBase<E> {
     protected String remoteAddr = null;
     protected String remoteHost = null;
     protected int remotePort = -1;
-
-    /**
+    /*
+     * Used if block/non-blocking is set at the socket level. The client is
+     * responsible for the thread-safe use of this field via the locks provided.
+     */
+    private volatile boolean blockingStatus = true;
+    private final Lock blockingStatusReadLock;
+    private final WriteLock blockingStatusWriteLock;
+    /*
      * Used to record the first IOException that occurs during non-blocking
      * read/writes that can't be usefully propagated up the stack since there is
      * no user code or appropriate container code in the stack to handle it.
@@ -93,17 +103,17 @@ public abstract class SocketWrapperBase<E> {
      */
     protected final WriteBuffer nonBlockingWriteBuffer = new WriteBuffer(bufferedWriteSize);
 
-    /*
-     * Asynchronous operations.
-     */
     protected final Semaphore readPending;
     protected volatile OperationState<?> readOperation = null;
     protected final Semaphore writePending;
     protected volatile OperationState<?> writeOperation = null;
 
-    public SocketWrapperBase(E socket, AbstractEndpoint<E> endpoint) {
+    public SocketWrapperBase(E socket, AbstractEndpoint<E,?> endpoint) {
         this.socket = socket;
         this.endpoint = endpoint;
+        ReentrantReadWriteLock lock = new ReentrantReadWriteLock();
+        this.blockingStatusReadLock = lock.readLock();
+        this.blockingStatusWriteLock = lock.writeLock();
         if (endpoint.getUseAsyncIO() || needSemaphores()) {
             readPending = new Semaphore(1);
             writePending = new Semaphore(1);
@@ -117,7 +127,11 @@ public abstract class SocketWrapperBase<E> {
         return socket;
     }
 
-    public AbstractEndpoint<E> getEndpoint() {
+    protected void reset(E closedSocket) {
+        socket = closedSocket;
+    }
+
+    protected AbstractEndpoint<E,?> getEndpoint() {
         return endpoint;
     }
 
@@ -198,9 +212,13 @@ public abstract class SocketWrapperBase<E> {
         return this.writeTimeout;
     }
 
+    public void setKeepAliveLeft(int keepAliveLeft) {
+        this.keepAliveLeft = keepAliveLeft;
+    }
 
-    public void setKeepAliveLeft(int keepAliveLeft) { this.keepAliveLeft = keepAliveLeft;}
-    public int decrementKeepAlive() { return (--keepAliveLeft);}
+    public int decrementKeepAlive() {
+        return --keepAliveLeft;
+    }
 
     public String getRemoteHost() {
         if (remoteHost == null) {
@@ -250,6 +268,14 @@ public abstract class SocketWrapperBase<E> {
     }
     protected abstract void populateLocalPort();
 
+    public boolean getBlockingStatus() { return blockingStatus; }
+    public void setBlockingStatus(boolean blockingStatus) {
+        this.blockingStatus = blockingStatus;
+    }
+    public Lock getBlockingStatusReadLock() { return blockingStatusReadLock; }
+    public WriteLock getBlockingStatusWriteLock() {
+        return blockingStatusWriteLock;
+    }
     public SocketBufferHandler getSocketBufferHandler() { return socketBufferHandler; }
 
     public boolean hasDataToRead() {
@@ -360,8 +386,27 @@ public abstract class SocketWrapperBase<E> {
     }
 
 
-    public abstract void close() throws IOException;
-    public abstract boolean isClosed();
+    /**
+     * Close the socket wrapper.
+     */
+    public void close() {
+        if (closed.compareAndSet(false, true)) {
+            doClose();
+        }
+    }
+
+    /**
+     * Perform the actual close. The closed atomic boolean guarantees this will
+     * be called only once per wrapper.
+     */
+    protected abstract void doClose();
+
+    /**
+     * @return true if the wrapper has been closed
+     */
+    public boolean isClosed() {
+        return closed.get();
+    }
 
 
     /**
@@ -476,17 +521,14 @@ public abstract class SocketWrapperBase<E> {
      * @throws IOException If an IO error occurs during the write
      */
     protected void writeBlocking(byte[] buf, int off, int len) throws IOException {
-        if (len > 0) {
+        socketBufferHandler.configureWriteBufferForWrite();
+        int thisTime = transfer(buf, off, len, socketBufferHandler.getWriteBuffer());
+        while (socketBufferHandler.getWriteBuffer().remaining() == 0) {
+            len = len - thisTime;
+            off = off + thisTime;
+            doWrite(true);
             socketBufferHandler.configureWriteBufferForWrite();
-            int thisTime = transfer(buf, off, len, socketBufferHandler.getWriteBuffer());
-            len -= thisTime;
-            while (len > 0) {
-                off += thisTime;
-                doWrite(true);
-                socketBufferHandler.configureWriteBufferForWrite();
-                thisTime = transfer(buf, off, len, socketBufferHandler.getWriteBuffer());
-                len -= thisTime;
-            }
+            thisTime = transfer(buf, off, len, socketBufferHandler.getWriteBuffer());
         }
     }
 
@@ -505,14 +547,48 @@ public abstract class SocketWrapperBase<E> {
      * @throws IOException If an IO error occurs during the write
      */
     protected void writeBlocking(ByteBuffer from) throws IOException {
-        if (from.hasRemaining()) {
+        if (socketBufferHandler.isWriteBufferEmpty()) {
+            // Socket write buffer is empty. Write the provided buffer directly
+            // to the network.
+            // TODO Shouldn't smaller writes be buffered anyway?
+            writeBlockingDirect(from);
+        } else {
+            // Socket write buffer has some data.
+            socketBufferHandler.configureWriteBufferForWrite();
+            // Put as much data as possible into the write buffer
+            transfer(from, socketBufferHandler.getWriteBuffer());
+            // If the buffer is now full, write it to the network and then write
+            // the remaining data directly to the network.
+            if (!socketBufferHandler.isWriteBufferWritable()) {
+                doWrite(true);
+                writeBlockingDirect(from);
+            }
+        }
+    }
+
+
+    /**
+     * Writes directly to the network, bypassing the socket write buffer.
+     *
+     * @param from The ByteBuffer containing the data to be written
+     *
+     * @throws IOException If an IO error occurs during the write
+     */
+    protected void writeBlockingDirect(ByteBuffer from) throws IOException {
+        // The socket write buffer capacity is socket.appWriteBufSize
+        // TODO This only matters when using TLS. For non-TLS connections it
+        //      should be possible to write the ByteBuffer in a single write
+        int limit = socketBufferHandler.getWriteBuffer().capacity();
+        int fromLimit = from.limit();
+        while (from.remaining() >= limit) {
+            from.limit(from.position() + limit);
+            doWrite(true, from);
+            from.limit(fromLimit);
+        }
+
+        if (from.remaining() > 0) {
             socketBufferHandler.configureWriteBufferForWrite();
             transfer(from, socketBufferHandler.getWriteBuffer());
-            while (from.hasRemaining()) {
-                doWrite(true);
-                socketBufferHandler.configureWriteBufferForWrite();
-                transfer(from, socketBufferHandler.getWriteBuffer());
-            }
         }
     }
 
@@ -535,12 +611,11 @@ public abstract class SocketWrapperBase<E> {
      * @throws IOException If an IO error occurs during the write
      */
     protected void writeNonBlocking(byte[] buf, int off, int len) throws IOException {
-        if (len > 0 && nonBlockingWriteBuffer.isEmpty()
-                && socketBufferHandler.isWriteBufferWritable()) {
+        if (nonBlockingWriteBuffer.isEmpty() && socketBufferHandler.isWriteBufferWritable()) {
             socketBufferHandler.configureWriteBufferForWrite();
             int thisTime = transfer(buf, off, len, socketBufferHandler.getWriteBuffer());
-            len -= thisTime;
-            while (len > 0) {
+            len = len - thisTime;
+            while (!socketBufferHandler.isWriteBufferWritable()) {
                 off = off + thisTime;
                 doWrite(false);
                 if (len > 0 && socketBufferHandler.isWriteBufferWritable()) {
@@ -552,7 +627,7 @@ public abstract class SocketWrapperBase<E> {
                     // else to do here. Exit the loop.
                     break;
                 }
-                len -= thisTime;
+                len = len - thisTime;
             }
         }
 
@@ -581,12 +656,11 @@ public abstract class SocketWrapperBase<E> {
     protected void writeNonBlocking(ByteBuffer from)
             throws IOException {
 
-        if (from.hasRemaining() && nonBlockingWriteBuffer.isEmpty()
-                && socketBufferHandler.isWriteBufferWritable()) {
+        if (nonBlockingWriteBuffer.isEmpty() && socketBufferHandler.isWriteBufferWritable()) {
             writeNonBlockingInternal(from);
         }
 
-        if (from.hasRemaining()) {
+        if (from.remaining() > 0) {
             // Remaining data must be buffered
             nonBlockingWriteBuffer.add(from);
         }
@@ -602,20 +676,47 @@ public abstract class SocketWrapperBase<E> {
      * @throws IOException If an IO error occurs during the write
      */
     protected void writeNonBlockingInternal(ByteBuffer from) throws IOException {
-        socketBufferHandler.configureWriteBufferForWrite();
-        transfer(from, socketBufferHandler.getWriteBuffer());
-        while (from.hasRemaining()) {
-            doWrite(false);
-            if (socketBufferHandler.isWriteBufferWritable()) {
-                socketBufferHandler.configureWriteBufferForWrite();
-                transfer(from, socketBufferHandler.getWriteBuffer());
-            } else {
-                break;
+        if (socketBufferHandler.isWriteBufferEmpty()) {
+            writeNonBlockingDirect(from);
+        } else {
+            socketBufferHandler.configureWriteBufferForWrite();
+            transfer(from, socketBufferHandler.getWriteBuffer());
+            if (!socketBufferHandler.isWriteBufferWritable()) {
+                doWrite(false);
+                if (socketBufferHandler.isWriteBufferWritable()) {
+                    writeNonBlockingDirect(from);
+                }
             }
         }
     }
 
 
+    protected void writeNonBlockingDirect(ByteBuffer from) throws IOException {
+        // The socket write buffer capacity is socket.appWriteBufSize
+        // TODO This only matters when using TLS. For non-TLS connections it
+        //      should be possible to write the ByteBuffer in a single write
+        int limit = socketBufferHandler.getWriteBuffer().capacity();
+        int fromLimit = from.limit();
+        while (from.remaining() >= limit) {
+            int newLimit = from.position() + limit;
+            from.limit(newLimit);
+            doWrite(false, from);
+            from.limit(fromLimit);
+            if (from.position() != newLimit) {
+                // Didn't write the whole amount of data in the last
+                // non-blocking write.
+                // Exit the loop.
+                return;
+            }
+        }
+
+        if (from.remaining() > 0) {
+            socketBufferHandler.configureWriteBufferForWrite();
+            transfer(from, socketBufferHandler.getWriteBuffer());
+        }
+    }
+
+
     /**
      * Writes as much data as possible from any that remains in the buffers.
      *
@@ -911,7 +1012,6 @@ public abstract class SocketWrapperBase<E> {
         protected final CompletionHandler<Long, ? super A> handler;
         protected final Semaphore semaphore;
         protected final VectoredIOCompletionHandler<A> completion;
-        protected final AtomicBoolean callHandler;
         protected OperationState(boolean read, ByteBuffer[] buffers, int offset, int length,
                 BlockingMode block, long timeout, TimeUnit unit, A attachment,
                 CompletionCheck check, CompletionHandler<Long, ? super A> handler,
@@ -928,7 +1028,6 @@ public abstract class SocketWrapperBase<E> {
             this.handler = handler;
             this.semaphore = semaphore;
             this.completion = completion;
-            callHandler = (handler != null) ? new AtomicBoolean(true) : null;
         }
         protected volatile long nBytes = 0;
         protected volatile CompletionState state = CompletionState.PENDING;
@@ -1011,7 +1110,7 @@ public abstract class SocketWrapperBase<E> {
                         state.state = currentState;
                     }
                     state.end();
-                    if (completion && state.handler != null && state.callHandler.compareAndSet(true, false)) {
+                    if (completion && state.handler != null) {
                         state.handler.completed(Long.valueOf(state.nBytes), state.attachment);
                     }
                     synchronized (state) {
@@ -1052,7 +1151,7 @@ public abstract class SocketWrapperBase<E> {
                 state.state = state.isInline() ? CompletionState.ERROR : CompletionState.DONE;
             }
             state.end();
-            if (state.handler != null && state.callHandler.compareAndSet(true, false)) {
+            if (state.handler != null) {
                 state.handler.failed(exc, state.attachment);
             }
             synchronized (state) {
@@ -1382,15 +1481,10 @@ public abstract class SocketWrapperBase<E> {
                     try {
                         state.wait(unit.toMillis(timeout));
                         if (state.state == CompletionState.PENDING) {
-                            if (handler != null && state.callHandler.compareAndSet(true, false)) {
-                                handler.failed(new SocketTimeoutException(), attachment);
-                            }
                             return CompletionState.ERROR;
                         }
                     } catch (InterruptedException e) {
-                        if (handler != null && state.callHandler.compareAndSet(true, false)) {
-                            handler.failed(new SocketTimeoutException(), attachment);
-                        }
+                        completion.failed(new SocketTimeoutException(), state);
                         return CompletionState.ERROR;
                     }
                 }
@@ -1425,13 +1519,4 @@ public abstract class SocketWrapperBase<E> {
         }
         return max;
     }
-
-    protected static boolean buffersArrayHasRemaining(ByteBuffer[] buffers, int offset, int length) {
-        for (int pos = offset; pos < offset + length; pos++) {
-            if (buffers[pos].hasRemaining()) {
-                return true;
-            }
-        }
-        return false;
-    }
 }
