@@ -18,6 +18,7 @@ package org.apache.tomcat.websocket;
 
 import java.net.URI;
 import java.nio.ByteBuffer;
+import java.nio.charset.StandardCharsets;
 import java.util.concurrent.CountDownLatch;
 import java.util.concurrent.TimeUnit;
 
@@ -39,8 +40,7 @@ import org.apache.tomcat.websocket.TesterMessageCountClient.TesterProgrammaticEn
 
 public class TestWsPingPongMessages extends WebSocketBaseTest {
 
-    ByteBuffer applicationData = ByteBuffer.wrap(new String("mydata")
-            .getBytes());
+    ByteBuffer applicationData = ByteBuffer.wrap("mydata".getBytes(StandardCharsets.UTF_8));
 
     @Test
     public void testPingPongMessages() throws Exception {
